# LusiApi

A Ruby client for Lancaster University's LUSI API for student record management.

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'lusi_api'
```

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install lusi_api

## Usage

### Basic usage

Create an API instance with your LUSI account credentials:

```ruby
require 'lusi_api'

api = LUSI::API::Core::API.new(api_user: 'Your LUSI username', api_password: 'Your LUSI password')
```

Then pass this as the first argumnent of the ```get_instance``` factory methods of the API data classes.

### Common classes

All LUSI API classes are subclasses of ```LUSI::API::Core::Object.```

Entities with identity are subclasses of ```LUSI::API::Core::ObjectWithID.``` Instances use the LUSI ID for equality and
hashing.

Properties consisting of an id/name pair are instances of ```LUSI::API::Core::Code.```

### Lookup

The LUSI web API frequently provides codes to represent linked or embedded objects (for example, the department a
student is affiliated with). To assist with resolving codes to objects, the LUSI API provides the
```LUSI::API::Core::Lookup``` module.

#### LookupTable

The ```LUSI::API::Core::Lookup::LookupTable``` class extends Ruby's ```Hash``` so that failed key lookups trigger an
attempt to read the value from an external data source by calling the ```get_value(key)``` method.

Subclasses implement ```get_value``` to perform the key lookup in the data source. If the lookup succeeds, the
corresponding value should be returned. If it fails, ```LUSI::API::Core::Lookup::LookupError``` should be raised.

If ```get_value``` returns a value, this value is added to the lookup table hash and returned as the result of the 
lookup. If ```LookupError``` is raised, the usual ```Hash``` handling of a failed lookup is invoked (default value or
code block).

#### LUSILookupTable

The ```LUSI::API::Core::Lookup::LUSILookupTable``` class extends ```LookupTable``` to retrieve data from the LUSI web
API when a key lookup fails.

```ruby
require 'lusi_api/core/lookup'

# The API class of the objects to be added to the lookup table
result_class = LUSI::API::Core::Code

# Positional arguments to the result class' get_instance method
args = ['arg1', 'arg2']

# Keyword arguments to the result class' get_instance method
kwargs = { arg1: 'value1', arg2: 'value2' }

# The parameter of the result_class' get_instance method used as the search condition
param = 'identity'

# The attribute of the result_class instance supplying the key value
# This may be a String or Symbol (the name of the attribute) or a Proc (accepting the result_class instance as its
# single argument and returning the key value). If not specified, the result_class instance is expected to have an
# attribute named 'identity' which supplies the key value.
# NameError is raised if the attribute lookup fails.
key_attribute = 'object_id'

begin
  lookup = LUSI::API::Core::Lookup::LUSILookupTable(api, result_class, *args,
                                                    key_attribute: key_attribute,
                                                    param: param,
                                                    **kwargs)
rescue NameError
  # The key attribute lookup failed
end 
```

Some lookup tables (for example, academic weeks, countries) can be populated from a single call to the LUSI web API.
```LUSILookupTable``` provides the ```load``` method to support this.

A lookup table which supports bulk loading should include the constructor parameter ```loadable: true``` and may
specify an optional hash of default parameter values for the ```load``` method:
 
```ruby
begin
  lookup = LUSI::API::Core::Lookup::LUSILookupTable(api, result_class, ...,
                                                    loadable: true,
                                                    load_params: {})
end
```
  
 
### Exceptions

All exceptions thrown by the LUSI API are subclasses of ```LUSI::API::Core::APIError.```

The exception hiearchy is:
```
 APIError
     APICallError
         APITimeoutError  
     APIResponseError   
         APIContentError
         APIPermissionError
```

Methods which interact with the LUSI web API, such as the ```get_instance``` method of the data classes, should
rescue at least ```LUSI::API::Core::APIError``` to handle network problems.

```ruby
begin
  result = SomeClass.get_instance(api)
rescue LUSI::API::Core::APITimeoutError
  # Subclass of APICallError 
  # Raised when the LUSI API does not respond within the timeout period
rescue LUSI::API::Core::APICallError
  # Subclass of APIError
  # Raised when there is a general problem with the LUSI API call (e.g. network error)
rescue LUSI::API::Core::APIContentError
  # Subclass of APIResponseError
  # Raised when there is a problem parsing the XML response from the LUSI API
rescue LUSI::API::Core::APIPermissionError
  # Subclass of APIResponseError
  # Raised when the LUSI account does not have permission to call the requested API method
rescue LUSI::API::Core::APIError
  # Raised when any other problem occurs
end
```

### Calendar module

The ```LUSI::API::Calendar``` module provides the classes ```Week``` and ```Year``` to represent LUSI academic week and
year objects. Both classes extend ```LUSI::API::Core::Code.```

#### Year

Years in LUSI are objects with an 6-digit zero-padded identity code derived from the year: 000nnn
where nnn = year - 1900.

e.g. 2016 = 2016 - 1900 = 000116
  
```ruby
require 'lusi_api/calendar'

# Get an array of all years in LUSI
all_years = LUSI::API::Calendar::Year.get_instance(api, lookup)
year_2016 = all_years.select { |year| year.identity == '000116' }

# Year properties
puts <<EOF
  Identity: #{year_2016.identity}
  Description: #{year_2016.description}
  Full year description: #{year_2016.full_year}
EOF
```

#### Week

```ruby
require 'lusi_api/calendar'

# Get an array of all weeks in LUSI
all_weeks = LUSI::API::Calendar::Week.get_instance(api, lookup)

# Get an array of all weeks for a specific year
# year may be a year identity string or a LUSI::API::Calendar::Year instance
year_weeks = LUSI::API::Calendar::Week.get_instance(api, lookup, year_identity: '000116')

# Get the current academic week
current_week = LUSI::API::Calendar::Week.current_academic_week(api)

# Week properties
# - monday_date is an instance of DateTime
# - year is an instance of LUSI::API::Calendar::Year
puts <<EOF
  Identity: #{current_week.identity}
  Description: #{current_week.description}
  Academic timetable week ID: #{current_week.att_identity}
  Monday date: #{current_week.monday_date}
  Term: #{current_week.term}
  Year: #{current_week.year}
EOF
```


### Country module

### Course module

### Enrolment module

### Organisation module

### Person module

### Service account module

### Service method module

## Development

After checking out the repo, run `bin/setup` to install dependencies. You can also run `bin/console` for an interactive prompt that will allow you to experiment.

To install this gem onto your local machine, run `bundle exec rake install`. To release a new version, update the version number in `version.rb`, and then run `bundle exec rake release`, which will create a git tag for the version, push git commits and tags, and push the `.gem` file to [rubygems.org](https://rubygems.org).

## Contributing

Bug reports and pull requests are welcome on GitHub at https://github.com/[USERNAME]/lusi_api.


## License

The gem is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).

